#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Wrester for obtaining metadata for a given spl observation for UI display.
"""
import json

# pylint: disable=E0401, E0402, R0903
import logging

import pendulum
from aat_wrest.utilities import TIME_ZONE, MDDBConnector, format_interval


class WrestObservationMetadata:
    """
    Class for extracting observation metadata
    """

    def __init__(self, connection: MDDBConnector, sdm_id: str = None, spl: str = None):
        self.logger = logging.getLogger("aat_wrest")
        self.conn = connection
        self.sdm_id = sdm_id
        self.spl = spl

    def wrest_observation_info(self) -> json:
        """
        Given a sdm_id, query the Metadata DB for observation data related to an ingestion-complete event

        :param sdm_id:
        :return science_product_locator:
        """
        metadata = {}
        sql = f"""
        SELECT e.science_product_locator,
         e.band_code as band_code,
         e.configuration as configuration,
         e.starttime as starttime,
         e.endtime as endtime,
         sp.is_srdp as is_srdp
         FROM execution_blocks e
            JOIN science_products sp on e.science_product_locator = sp.science_product_locator
         WHERE ngas_fileset_id = %(sdm_id)s
        """
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"sdm_id": self.sdm_id})
            data = cursor.fetchall()
            if data:
                metadata = json.dumps(
                    {
                        "spl": data[0][0],
                        "bands": data[0][1],
                        "array_config": data[0][2],
                        "obs_start_time": data[0][3],
                        "obs_end_time": data[0][4],
                        "is_srdp": data[0][5],
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for SDM id {self.sdm_id}"
                )
        finally:
            self.conn.close()
        return metadata

    def wrest_observation_info_via_spl(self) -> json:
        """
        Given a science_product_locator, query the Metadata DB for observation data related to an ingestion-complete event

        :param science_product_locator:
        :return metadata:
        """
        metadata = {}
        sql = f"""
            SELECT e.science_product_locator,
            e.band_code as band_code,
            e.configuration as configuration,
            e.starttime as starttime,
            e.endtime as endtime,
            e.ngas_fileset_id as ngas_fileset_id,
            sp.is_srdp as is_srdp
            FROM execution_blocks e
                JOIN science_products sp on e.science_product_locator = sp.science_product_locator
            WHERE e.science_product_locator = %(spl)s
            """
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"spl": self.spl})
            data = cursor.fetchall()
            if data:
                metadata = json.dumps(
                    {
                        "spl": data[0][0],
                        "bands": data[0][1],
                        "array_config": data[0][2],
                        "obs_start_time": data[0][3],
                        "obs_end_time": data[0][4],
                        "sdm_id": data[0][5],
                        "is_srdp": data[0][6],
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for SPL {self.spl}"
                )
        finally:
            self.conn.close()
        return metadata
